package com.zbsz.controller;

import com.common.entity.Scene;
import com.common.entity.dic.IndicatorDic;
import com.common.entity.dic.AppDic;
import com.common.info.CodeMsg;
import com.zbsz.entity.Indicator;
import com.zbsz.info.Result;
import com.zbsz.repository.IndicatorDicRepository;
import com.zbsz.repository.AppDicRepository;
import com.zbsz.repository.IndicatorRepository;
import com.zbsz.repository.SceneRepository;
import com.zbsz.service.DynamicTableService;
import com.zbsz.service.SearchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/indicator")
@Api(tags = "服务接口")
public class IndicatorController {
    @Resource
    private SceneRepository sceneRepository;
    @Resource
    private IndicatorDicRepository indicatorDicRepository;
    @Resource
    private AppDicRepository appDicRepository;
    @Resource
    private IndicatorRepository indicatorRepository;
    @Resource
    private DynamicTableService dynamicTableService;
    @Resource
    private SearchService searchService;

    private void setSuccessInfo(Result result){
        result.setCode(CodeMsg.SUCCESS.getCode());
        result.setMsg(CodeMsg.SUCCESS.getMessage());
    }

    private void setSuccessFlagInfo(Result result, boolean flag){
        setSuccessInfo(result);
        result.setData(flag);
        result.setTotal(0L);
    }

    private void setSuccessListInfo(Result result, List list){
        setSuccessInfo(result);
        result.setData(list);
        if(list!=null){
            result.setTotal((long) list.size());
        }else {
            result.setTotal(0L);
        }
    }

    private void setFailureInfo(Result result){
        result.setCode(CodeMsg.FAILURE.getCode());
        result.setMsg(CodeMsg.FAILURE.getMessage());
        result.setTotal(0L);
    }

    private void setFailureEInfo(Result result, Exception e){
        result.setCode(CodeMsg.FAILURE.getCode());
        result.setMsg(e.getMessage());
        result.setTotal(0L);
    }

    /**
     * 关联场景查询
     * @return Result
     */
    @ApiOperation(value = "获取关联场景",notes = "获取关联场景列表信息")
    @GetMapping("get_scenes")
    public Result getScenes() {
        Result result = new Result();
        try {
            List<Scene> list = sceneRepository.list();
            setSuccessListInfo(result,list);
            log.info("-------- 获取关联场景成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联场景异常 : "+e.getMessage(),e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    /**
     * 指标类型查询
     * @return Result
     */
    @ApiOperation(value = "获取指标类型",notes = "获取指标类型列表信息")
    @GetMapping("get_indicator_dic")
    public Result getIndicatorDic() {
        Result result = new Result();
        try {
            List<IndicatorDic> list = indicatorDicRepository.list();
            setSuccessListInfo(result,list);
            log.info("-------- 获取指标类型成功 ");
        }catch (Exception e){
            log.error("-------- 获取指标类型异常 : "+e.getMessage(),e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    /**
     * 关联应用模块类型查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用模块类型",notes = "获取关联应用模块类型列表信息")
    @GetMapping("get_app_p_dic")
    public Result getAppPDic() {
        Result result = new Result();
        try {
            List<AppDic> list = appDicRepository.lambdaQuery().isNull(AppDic::getParentId).list();
            setSuccessListInfo(result,list);
            log.info("-------- 获取关联应用模块成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用模块异常 : "+e.getMessage(),e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    /**
     * 关联功能类型查询
     * @return Result
     */
    @ApiOperation(value = "获取联功能类型",notes = "获取联功能类型列表信息")
    @ApiImplicitParam(name = "p_id",value = "联应用模块类型 id，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("get_app_c_dic")
    public Result getAppCDic(Integer p_id) {
        Result result = new Result();
        try {
            List<AppDic> list = appDicRepository.lambdaQuery().eq(AppDic::getParentId,p_id).list();
            setSuccessListInfo(result,list);
            log.info("-------- 获取联功能成功 ");
        }catch (Exception e){
            log.error("-------- 获取联功能异常,p_id : "+p_id,e);
            setFailureEInfo(result,e);
        }
        return result;
    }


    /**
     * 关联应用表信息查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用表信息",notes = "获取关联应用表信息")
    @GetMapping("get_app_tables")
    public Result getAppTables() {
        Result result = new Result();
        try {
            List<String> list = searchService.getTableNames();
            setSuccessListInfo(result,list);
            log.info("-------- 获取关联应用表成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用表异常 : "+e.getMessage(),e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    /**
     * 关联应用表信息查询
     * @return Result
     */
    @ApiOperation(value = "获取关联应用表字段信息",notes = "获取关联应用表字段信息")
    @ApiImplicitParam(name = "tableName",value = "关联应用表名称，Content-Type:application/x-www-form-urlencoded",required = true)
    @GetMapping("get_app_table_columns")
    public Result getAppTableColumns(String tableName) {
        Result result = new Result();
        try {
            List<String> list = searchService.getTableColumns(tableName);
            setSuccessListInfo(result,list);
            log.info("-------- 获取关联应用表字段成功 ");
        }catch (Exception e){
            log.error("-------- 获取关联应用表字段异常 : "+e.getMessage(),e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    private boolean validEnName(Indicator indicator){
        List<Indicator> list= indicatorRepository.lambdaQuery()
                .eq(Indicator::getSceneId,indicator.getSceneId())
                .eq(Indicator::getIndicatorDicId,indicator.getIndicatorDicId())
                .eq(Indicator::getAppDicPId,indicator.getAppDicPId())
                .eq(Indicator::getAppDicCId,indicator.getAppDicCId())
                .eq(Indicator::getIndicatorEnName,indicator.getIndicatorEnName())
                .list();
        if(list!=null&&list.size()>0){
            return true;
        }else {
            return false;
        }
    }

    /**
     * 校验指标英文名称
     * @return Result
     */
    @ApiOperation(value = "校验指标英文名称",notes = "true：表示重名，false：表示可以保存")
    @GetMapping("valid_indicator_en_name")
    public Result validIndicatorEnName(@RequestBody Indicator indicator) {
        Result result = new Result();
        try {
            boolean flag = validEnName(indicator);
            setSuccessFlagInfo(result,flag);
            if(flag){
                log.error("-------- 指标英文名称重名！！！ name :"+indicator.getIndicatorEnName());
            }else {
                log.info("-------- 指标英文名称可用,name :"+indicator.getIndicatorEnName());
            }
        }catch (Exception e){
            log.error("-------- 获取联功能异常,indicator : "+indicator,e);
            setFailureEInfo(result,e);
        }
        return result;
    }


    /**
     * 保存修改指标
     * @return Result
     */
    @ApiOperation(value = "保存指标",notes = "保存指标")
    @PostMapping("save_update_indicator")
    @Transactional
    public Result saveUpdateIndicator(@RequestBody Indicator indicator) {
        Result result = new Result();
        try {
            //1、校验指标名称会不会重复
            boolean enNameFlag = validEnName(indicator);
            if(enNameFlag){
                setFailureInfo(result);
                result.setData("指标英文名称重名， name :"+indicator.getIndicatorEnName());
                log.error("-------- 指标英文名称重名！！！ name :"+indicator.getIndicatorEnName());
                return result;
            }

            boolean save_indicator_flag = indicatorRepository.saveOrUpdate(indicator);
            boolean save_dynamic_table_flag = true;
            if(save_indicator_flag){
                switch (indicator.getAppDicPId()) {
                    //智能战果分析时，创建动态表
                    case 2:
                        save_dynamic_table_flag = dynamicTableService.createTable(indicator);
                        break;
                }
                if(save_dynamic_table_flag){
                    setSuccessFlagInfo(result,true);
                    log.info("-------- 保存指标成功 ");
                }else {
                    result.setData("动态创建表失败");
                    setFailureInfo(result);
                    log.error("-------- 动态创建表异常,indicator : "+indicator);
                }

            }else {
                result.setData("保存指标失败");
                setFailureInfo(result);
                log.error("-------- 保存指标异常,indicator : "+indicator);
            }
        }catch (Exception e){
            log.error("-------- 保存指标异常,indicator : "+indicator,e);
            setFailureEInfo(result,e);
        }
        return result;
    }

    /**
     * 删除指标
     * @return Result
     */
    @ApiOperation(value = "删除指标",notes = "删除指标")
    @PostMapping("delete_indicator")
    @Transactional
    public Result delete_Indicator(@RequestBody Indicator indicator) {
        Result result = new Result();
        try {
            boolean delete_indicator_flag = indicatorRepository.removeById(indicator);
            boolean delete_dynamic_table_flag = true;
            if(delete_indicator_flag){
                switch (indicator.getAppDicPId()) {
                    //智能战果分析时，创建动态表
                    case 2:
                        delete_dynamic_table_flag = dynamicTableService.deleteTable(indicator);
                        break;
                }
                if(delete_dynamic_table_flag){
                    setSuccessFlagInfo(result,true);
                    log.info("-------- 删除指标成功 ");
                }else {
                    result.setData("动态删除表失败");
                    setFailureInfo(result);
                    log.error("-------- 动态删除表异常,indicator : "+indicator);
                }
            }else {
                result.setData("删除指标失败");
                setFailureInfo(result);
                log.error("-------- 删除指标异常,indicator : "+indicator);
            }
        }catch (Exception e){
            log.error("-------- 删除指标异常,indicator : "+indicator,e);
            setFailureEInfo(result,e);
        }
        return result;
    }




}
